(* eAGLContext.ml     Wrapper for Cocoa Touch EAGLContext
 *
 * Copyright (c) 2011 Psellos   http://psellos.com
 * Licensed under the MIT license:
 *     http://www.opensource.org/licenses/mit-license.php
 *)
type renderingAPI =
    | RenderingAPIOpenGLES1
    | RenderingAPIOpenGLES2

let int_of_api = function
    | RenderingAPIOpenGLES1 -> 1
    | RenderingAPIOpenGLES2 -> 2

class type _t =
object
    inherit Wrapper.t
    method initWithAPI' : renderingAPI -> _t
    method renderbufferStorage'fromDrawable' :
        GlFramebuffer.enumeration -> CaLayer.t -> bool
    method presentRenderbuffer' : GlFramebuffer.enumeration -> bool
end

external _initWithAPI_ : nativeint -> int -> _t =
    "EAGLContext_initWithAPI_"
external _renderbufferStorage_fromDrawable_ :
        nativeint -> int -> nativeint -> bool =
    "EAGLContext_renderbufferStorage_fromDrawable_"
external _presentRenderbuffer_ : nativeint -> int -> bool =
    "EAGLContext_presentRenderbuffer_"
external _setCurrentContext_ : nativeint -> bool =
    "EAGLContext_setCurrentContext_"

class t (robjcv: nativeint) =
object (self)
    inherit Wrapper.t robjcv

    method initWithAPI' api =
        _initWithAPI_ self#contents (int_of_api api)
    method renderbufferStorage'fromDrawable' targ (layer: CaLayer.t) =
        _renderbufferStorage_fromDrawable_
            self#contents (GlFramebuffer.code targ) layer#contents
    method presentRenderbuffer' targ =
        _presentRenderbuffer_ self#contents (GlFramebuffer.code targ)
end

(* Create an instance that doesn't wrap an ObjC object.  To create a
 * full fledged instance, (alloc ())#initWithAPI' ...
 *)
let alloc () = new t 0n

let setCurrentContext' ctx =
    _setCurrentContext_ ctx#contents

let () = Callback.register "EAGLContext.wrap" (new t)
